#!/bin/bash
#
# recent-files:bin/recent-files.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?

declare -gri DEFAULT_DAYS=1

function usage {
  cat <<EOF
Find modified files since n days and open it or open containing folder.

Usage:
  $( basename "$0" ) --help
  $( basename "$0" ) [--home DIR] [--days DAYS]

Options:

  -h,       --help      Display this messsage and exit.
  -H DIR,   --home DIR  Set alternative home directory,
                        if not set use: '${HOME}'
  -d DAYS,  --days DAYS Number of days, if not set use: ${DEFAULT_DAYS}


EOF
}
function build_list {
  local -r __home__="$1"
  local -i __days__="$2"

  echoinfo "$( printf "Analyse du dossier '%s' sur %d jour(s)" "${__home__}" "${__days__}" )"
  zenity --notification --window-icon="info" --text="Analyse en cours..."
  (
    cd "${__home__}" || return $?
    find . -type d \
      '(' \
        -name '.git' \
        -or -name '.metadata' \
        -or -name 'org.eclipse.core.runtime' \
        -or -path './.cache' \
        -or -path './.cinnamon' \
        -or -path './.config' \
        -or -path './.linuxmint' \
        -or -path './.local' \
        -or -path './.mozilla' \
        -or -path './.thunderbird' \
        -or -path './.var' \
      ')' \
      -prune -o -type f -mtime -"${__days__}" -print
  ) || return $?
}

function prepare_list {
  local line=
  while read -r line ; do
    if [ -n "${line}" ] ; then
      echo "${count}"
      basename "${line}"
      echo "${line}"
      ((count++))
    fi
  done
}

function show_list {
  local -r __list__="$1"

  local -i lines_count=0
  lines_count="$( wc -l <<<"${__list__}" )"

  echoinfo "$( printf "Trouvé %d fichier(s)" "${lines_count}" )"

  local -i count=1
  local -r __message__="List des derniers fichiers"
  local -r __ok_label__="Sélectionner"
  local -r __cancel_label__="Quitter"
  local -r __column_2__="Fichier"
  local -r __column_3__="Chemin complet"

  prepare_list <<<"${__list__}" | zenity --list --width=800 --height=450 \
    --title="${__title__}" \
    --text="${__message__}" \
    --ok-label="${__ok_label__}" \
    --cancel-label="${__cancel_label__}" \
    --hide-column 1 --column='' --column="${__column_2__}" --column="${__column_3__}" || return $?
}

function do_action {
  local -r __file__="$1"

  local -r __message__="Définir l'action sur le fichier"

  local -a actions=()
  actions+=( '1' '1' 'Ouvrir le fichier' )
  actions+=( '2' '2' 'Ouvrir le dossier')

  local anwser=
  anwser="$(
    set -x
    zenity --list --title="${__title__}" --text="${__message__}" --radiolist --hide-header --column='' --column='ID' --column='' --hide-column=2 "${actions[@]}"
  )" || return $?
  echodebug "anwser=${anwser}"

  case "${anwser}" in
    '1')
      xdg-open "${__file__}" || return $?
      ;;

    '2')
      xdg-open "$( dirname "${__file__}" )" || return $?
      ;;
  esac
}

function find_recent_files {
  local -r basedir="$1"
  local -ri days="$2"

  local list=
  list="$( build_list "${basedir}" "${days}" )" || return $?

  local -i anwser=
  anwser="$( show_list "${list}" )" || return $?

  if [ "${anwser}" -ne 0 ] ; then
    local filepath=
    filepath="$( sed "${anwser}q;d" <<<"${list}" )"
    echodebug "filepath='${filepath}'"

    local file=
    file="$( realpath "${basedir}/${filepath}" )"
    echodebug "file='${file}'"

    do_action "${file}" || return $?
  fi
}

function main {
  local param=
  local param_basedir=
  local param_days=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      '-H'|'--home')
        param_basedir="$( P_param "$@" )" && shift || return $?
        ;;

      '-d'|'--days')
        param_days="$( P_param "$@" )" && shift || return $?
        ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        return 1
        ;;
    esac
  done

  if [ -z "${param_basedir}" ] ; then
    param_basedir="${HOME}"
  fi
  if [ ! -d "${param_basedir}" ] ; then
    echofatal "Can't find directory: '${param_basedir}'"
    return 1
  fi

  local -gr __title__="Recherche des fichiers rescents"

  if [ -z "${param_days}" ] ; then
    local -r __message__="Nombre de jours:"

    param_days="$(
      zenity --entry --title="${__title__}" --text="${__message__}:" --entry-text="${DEFAULT_DAYS}"
    )" || return $?
  fi

  if [ -z "${param_days}" ] ; then
    param_days="${DEFAULT_DAYS}"
  fi
  local -i param_days_value="${param_days}"
  if [ "${param_days_value}" -lt 1 ] ; then
    echofatal "Number of days must be greater than 1: '${param_days}'(${param_days_value})"
  fi

  find_recent_files "${param_basedir}" "${param_days_value}" || return $?
}

main "$@" || exit $?
